package se.hirt.w1.mockup;

import java.io.IOException;
import java.util.Random;
import java.util.UUID;

import se.hirt.w1.PhysicalQuantity;
import se.hirt.w1.Sensor;

public class RandomMockupSensor implements Sensor {
	private final Random rnd = new Random();
	private final PhysicalQuantity physicalQuantity;
	private final String id;
	private final String unitString;
	private Number lastValue = new Double(25.0);
	
	public RandomMockupSensor(PhysicalQuantity physicalQuantity) {
		this.physicalQuantity = physicalQuantity;
		this.id = UUID.randomUUID().toString();
		this.unitString = physicalQuantity == PhysicalQuantity.Temperature ? "\u00B0C": "%";
	}

	@Override
	public String getID() {
		return id;
	}

	@Override
	public Number getValue() throws IOException {
		return physicalQuantity == PhysicalQuantity.Temperature ? getValue(-10,25) : getValue(0,100) ;
	}

	private synchronized Number getValue(double min, double max) {
		// only change 5% either way  per reading
		double maxChange = (max - min) * .05;
		double lowBound = Math.max(min, lastValue.doubleValue() - maxChange);
		double highBound = Math.min(max, lastValue.doubleValue() + maxChange);
		lastValue = rnd.nextDouble() * (highBound - lowBound) + lowBound;
		return lastValue;
	}

	@Override
	public PhysicalQuantity getPhysicalQuantity() {
		return physicalQuantity;
	}

	@Override
	public String getUnitString() {
		return unitString;
	}

	@Override
	public void dispose() {
		// TODO Auto-generated method stub
		
	}

}
